@@ -1,8 +1,10 @@
 #define _LARGEFILE64_SOURCE	/* required for GLIBC to enable stat64 and friends */
+#include "doassert.h"
 #include <errno.h>
 #include <signal.h>
 #include <stdlib.h>
 #include <limits.h>
+#include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
 #include <regex.h>
@@ -21,12 +23,17 @@
 #include <sys/loadavg.h>
 #endif
 #include <dirent.h>
+#include <ctype.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <fcntl.h>
 
 #include "version.h"
 #include "error.h"
 #include "mem.h"
 #include "mt.h"
 #include "term.h"
+#include "globals.h"
 #include "utils.h"
 
 
@@ -38,7 +45,7 @@ int find_path_max(void)
 	int path_max = pathconf("/", _PC_PATH_MAX);
 	if (path_max <= 0)
 	{
-		if (errno) error_exit("find_path_max: pathconf(_PC_PATH_MAX) failed\n");
+		if (errno) error_exit(TRUE, FALSE, "pathconf() failed\n");
 
 		path_max = 4096;
 	}
@@ -56,7 +63,7 @@ int myrand(int max)
 	return (int)(((double)max * (double)rand()) / (double)RAND_MAX);
 }
 
-ssize_t WRITE(int fd, char *whereto, size_t len)
+ssize_t WRITE(int fd, char *whereto, size_t len, char *for_whom)
 {
 	ssize_t cnt=0;
 
@@ -69,7 +76,7 @@ ssize_t WRITE(int fd, char *whereto, size_t len)
 		if (rc == -1)
 		{
 			if (errno != EINTR && errno != EAGAIN)
-				error_exit("WRITE: Problem writing to filedescriptor\n");
+				error_exit(TRUE, FALSE, "Problem writing to file descriptor while processing %s.\n", for_whom);
 		}
 		else if (rc == 0)
 		{
@@ -86,19 +93,9 @@ ssize_t WRITE(int fd, char *whereto, size_t len)
 	return cnt;
 }
 
-/** get_load
- * - in:      nothing
- * - returns: char *
- * this function returns a string containing the current load of the system
- * multitail is running on. format: %1.2f %1.2f %1.2f
- * the string is malloc()ed so the caller should free() things
- */
-char *get_load(void)
+void get_load_values(double *v1, double *v2, double *v3)
 {
 #if !defined(__UCLIBC__) && (defined(__FreeBSD__) || defined(linux) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__) || defined(__GNU__) || defined(__sun) || defined(sun))
-	double loadavg[3];
-	char *str = (char *)mymalloc(LOADAVG_STR_LEN, "get_load: loadavg string");
-
 #if defined(__GLIBC__) && ( __GLIBC__ < 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ < 2))
 	/* Older glibc doesn't have getloadavg() - use sysinfo() */
 	/* thanks to Ville Herva for this code! */
@@ -111,24 +108,25 @@ char *get_load(void)
 		 * calls start to fail, something must be
 		 * really wrong
 		 */
-		error_exit("get_load: sysinfo() failed\n");
+		error_exit(TRUE, FALSE, "sysinfo() failed\n");
 	}
 
-	loadavg[0] = (double)si.loads[0] / scale;
-	loadavg[1] = (double)si.loads[1] / scale;
-	loadavg[2] = (double)si.loads[2] / scale;
+	*v1 = (double)si.loads[0] / scale;
+	*v2 = (double)si.loads[1] / scale;
+	*v3 = (double)si.loads[2] / scale;
 #else
+	double loadavg[3];
 	if (getloadavg(loadavg, 3) == -1)
 	{
 		/* see comment on sysinfo() */
-		error_exit("get_load: getloadavg() failed\n");
+		error_exit(TRUE, FALSE, "getloadavg() failed\n");
 	}
+	*v1 = loadavg[0];
+	*v2 = loadavg[1];
+	*v3 = loadavg[2];
 #endif
-	snprintf(str, LOADAVG_STR_LEN, "%1.2f %1.2f %1.2f", loadavg[0], loadavg[1], loadavg[2]);
-
-	return str;
 #else
-	return mystrdup("", "get_load:");
+	*v1 = *v2 = *v3 = -1.0;
 #endif
 }
 
@@ -138,14 +136,16 @@ int get_vmsize(pid_t pid)
 #if defined(linux)
 	FILE *fh;
 	int path_max = find_path_max();
-	char *path = mymalloc(path_max, "path");
+	char *path = mymalloc(path_max);
+
+	assert(pid > 1);
 
 	snprintf(path, path_max, "/proc/%d/stat", pid);
 
 	fh = fopen(path, "r");
 	if (fh)
 	{
-		char *dummystr = mymalloc(path_max, "get_vmsize: temp string");
+		char *dummystr = mymalloc(path_max);
 		char dummychar;
 		int dummy;
 
@@ -163,6 +163,15 @@ int get_vmsize(pid_t pid)
 	return vmsize;
 }
 
+int mykillpg(pid_t pid, int sigtype)
+{
+#ifdef AIX
+	return kill(pid, sigtype);
+#else
+	return killpg(pid, sigtype);
+#endif
+}
+
 /** stop_process
  * - in:      int pid  pid of process
  * - returns: nothing
@@ -171,42 +180,41 @@ int get_vmsize(pid_t pid)
  * is send so the process gets the possibility to gracefully exit. if it doesn't do that
  * in 100 microseconds, it is terminated
  */
-void stop_process(int pid)
+void stop_process(pid_t pid)
 {
-	if (pid < 2)
-		return;
+	assert(pid > 1);
 
-	if (kill(pid, SIGTERM) == -1)
+	if (mykillpg(pid, SIGTERM) == -1)
 	{
 		if (errno != ESRCH)
-			error_exit("stop_process: problem stopping child-process (%d)!\n", pid);
+			error_exit(TRUE, FALSE, "Problem stopping child process with PID %d (SIGTERM).\n", pid);
 	}
 
 	usleep(1000);
 
 	/* process still exists? */
-	if (kill(pid, SIGTERM) == 0)
+	if (mykillpg(pid, SIGTERM) == 0)
 	{
 		/* sleep for a millisecond... */
 		usleep(1000);
 
 		/* ...and then really terminate the process */
-		if (kill(pid, SIGKILL) == -1)
+		if (mykillpg(pid, SIGKILL) == -1)
 		{
 			if (errno != ESRCH)
-				error_exit("stop_process: problem killing child-process (%d)!\n", pid);
+				error_exit(TRUE, FALSE, "Problem stopping child process with PID %d (SIGKILL).\n", pid);
 		}
 	}
 	else if (errno != ESRCH)
-		error_exit("stop_process: problem stopping child-process (%d)!\n", pid);
+		error_exit(TRUE, FALSE, "Problem stopping child process with PID %d (SIGTERM).\n", pid);
 
-	/* wait for the last remainder of the exited process to go away,
+	/* wait for the last remainder of the died process to go away,
 	 * otherwhise we'll find zombies on our way
 	 */
 	if (waitpid(pid, NULL, WNOHANG | WUNTRACED) == -1)
 	{
 		if (errno != ECHILD)
-			error_exit("stop_process: waitpid failed\n");
+			error_exit(TRUE, FALSE, "waitpid() failed\n");
 	}
 }
 
@@ -221,6 +229,8 @@ void delete_array(char **list, int n)
 {
 	int loop;
 
+	assert(n >= 0);
+
 	for(loop=n-1; loop>=0; loop--)
 		myfree(list[loop]);
 
@@ -229,7 +239,11 @@ void delete_array(char **list, int n)
 
 int find_char_offset(char *str, char what)
 {
-	char *pnt = strchr(str, what);
+	char *pnt;
+
+	assert(what > 0);
+
+	pnt = strchr(str, what);
 	if (!pnt)
 		return -1;
 
@@ -243,12 +257,13 @@ int file_info(char *filename, off64_t *file_size, time_field_t tft, time_t *ts,
 	if (stat64(filename, &buf) == -1)
 	{
 		if (errno != ENOENT)
-			error_exit("file_info: error while doing stat() on file %s\n", filename);
+			error_exit(TRUE, FALSE, "Error while obtaining details of file %s.\n", filename);
 
 		return -1;
 	}
 
-	if (file_size) *file_size = buf.st_size;
+	if (file_size)
+		*file_size = buf.st_size;
 
 	if (ts)
 	{
@@ -258,20 +273,28 @@ int file_info(char *filename, off64_t *file_size, time_field_t tft, time_t *ts,
 			*ts = buf.st_mtime;
 		else if (tft == TT_CTIME)
 			*ts = buf.st_ctime;
-		else if (tft != 0)
-			error_exit("file_info: invalid timefield (%d)\n", (int)tft);
+		else
+		{
+			assert(tft == 0);
+			*ts = 0;
+		}
 	}
 
-	if (mode) *mode = buf.st_mode;
+	if (mode)
+		*mode = buf.st_mode;
 
 	return 0;
 }
 
 int file_exist(char *filename)
 {
-	struct stat64 buf;
+	struct stat buf;
+	int rc = stat(filename, &buf);
 
-	return stat64(filename, &buf);
+	if (rc == -1 && errno != ENOENT)
+		error_exit(TRUE, FALSE, "stat() on file %s failed.\n", filename);
+
+	return rc;
 }
 
 char * convert_regexp_error(int error, const regex_t *preg)
@@ -282,9 +305,11 @@ char * convert_regexp_error(int error, const regex_t *preg)
 	char *error_out = NULL;
 	const int max_err_len = 256;
 
-	if (error != 0 && error != REG_NOMATCH)
+	assert(error != 0);
+
+	if (error != REG_NOMATCH)
 	{
-		error_out = (char *)mymalloc(max_err_len + len + 1, "convert_regexp_error: regexp error string");
+		error_out = (char *)mymalloc(max_err_len + len + 1);
 
 		memcpy(error_out, multitail_string, len);
 
@@ -295,23 +320,11 @@ char * convert_regexp_error(int error, const regex_t *preg)
 	return error_out;
 }
 
-/* I'm sorry for this: */
-char check_date(void)
-{
-	time_t now = time(NULL);
-	struct tm *ptm = localtime(&now);
-
-	if (ptm -> tm_mon == (4 - 1) && ptm -> tm_mday == 2) /* April the 2nd? */
-	{
-		return 1;
-	}
-
-	return 0;
-}
-
 char * amount_to_str(long long int amount)
 {
-	char *out = mymalloc(AMOUNT_STR_LEN, "amount_to_str: converted amount string");	/* ...XB\0 */
+	char *out = mymalloc(AMOUNT_STR_LEN);	/* ...XB\0 */
+
+	assert(amount >= 0);
 
 	if (amount >= M_GB)	/* GB */
 		snprintf(out, AMOUNT_STR_LEN, "%dGB", (int)((amount + M_GB - 1) / M_GB));
@@ -325,31 +338,28 @@ char * amount_to_str(long long int amount)
 	return out;
 }
 
-char * replace_string(char *in, int pos_start, int pos_end, char *with_what)
-{
-	int str_len_in = strlen(in);
-	int str_len_ww = strlen(with_what);
-	int n_remove = (pos_end - pos_start) + 1; /* +1 => including pos_end! */
-	int new_len = str_len_in + str_len_ww - n_remove;
-	char *out = mymalloc(new_len + 1, "replace_string: new string"); /* +1 => 0x00 */
-
-	memcpy(out, in, pos_start);
-	memcpy(&out[pos_start], with_what, str_len_ww);
-	memcpy(&out[pos_start + str_len_ww], &in[pos_end + 1], str_len_in - (pos_end + 1));
-	out[new_len] = 0x00;
-
-	return out;
-}
-
 struct passwd *getuserinfo(void)
 {
 	struct passwd *pp = getpwuid(geteuid());
 	if (!pp)
-		error_exit("getuserinfo: failed to get passwdstructure for effective user id %d\n", geteuid());
+		error_exit(TRUE, FALSE, "Failed to get passwd-structure for effective user id %d.\n", geteuid());
 
 	return pp;
 }
 
+char * getusername(void)
+{
+	static char username[128] = { 0 };
+
+	if (!username[0])
+	{
+		strncpy(username, getuserinfo() -> pw_name, sizeof(username));
+		username[sizeof(username) - 1] = 0x00;
+	}
+
+	return username;
+}
+
 /* these are there because AIX/IRIX can return EINTR for those */
 int myopen(char *path, int mode)
 {
@@ -394,6 +404,8 @@ char * shorten_filename(char *in, int max_len)
 	int len = strlen(in);
 	int cutlen, dummy;
 
+	assert(max_len >= 0);
+
 	if (len <= max_len)
 		return in;
 
@@ -414,7 +426,7 @@ double get_ts(void)
 	struct timezone tz;
 
 	if (gettimeofday(&ts, &tz) == -1)
-		error_exit("get_ts: gettimeofday failed");
+		error_exit(TRUE, FALSE, "gettimeofday() failed");
 
 	return (((double)ts.tv_sec) + ((double)ts.tv_usec)/1000000.0);
 }
@@ -423,7 +435,7 @@ int match_files(char *search_for, char **path, char ***found, char **isdir)
 {
 	DIR *dir;
 	struct dirent *entry;
-	char *cur_dir = mymalloc(find_path_max() + 1, "match_files: current file");
+	char *cur_dir = mymalloc(find_path_max() + 1);
 	char *fname;
 	char **list = NULL;
 	int nfound = 0;
@@ -433,14 +445,14 @@ int match_files(char *search_for, char **path, char ***found, char **isdir)
 	char *slash = strrchr(search_for, '/');
 	if (slash)
 	{
-		fname = mystrdup(slash + 1, "match_files: filename");
+		fname = mystrdup(slash + 1);
 		*(slash + 1) = 0x00;
-		*path = mystrdup(search_for, "match_files: current path");
+		*path = mystrdup(search_for);
 	}
 	else
 	{
-		*path = mystrdup("./", "match_files: current path");
-		fname = mystrdup(search_for, "match_files: filename");
+		*path = mystrdup("./");
+		fname = mystrdup(search_for);
 	}
 	fname_size = strlen(fname);
 	path_len = strlen(*path);
@@ -462,16 +474,16 @@ int match_files(char *search_for, char **path, char ***found, char **isdir)
 			struct stat finfo;
 
 			/* get filename */
-			list = (char **)myrealloc(list, (nfound + 1) * sizeof(char *), "match_files: directory list");
-			list[nfound] = mystrdup(entry -> d_name, "match_files dir entry");
+			list = (char **)myrealloc(list, (nfound + 1) * sizeof(char *));
+			list[nfound] = mystrdup(entry -> d_name);
 
 			/* check if the file is a directory */
-			*isdir = (char *)myrealloc(*isdir, (nfound + 1) * sizeof(char), "ismatch_files: -dir list");
+			*isdir = (char *)myrealloc(*isdir, (nfound + 1) * sizeof(char));
 			strncpy(&cur_dir[path_len], entry -> d_name, max(0,find_path_max() - path_len));
 			if (stat(cur_dir, &finfo) == -1)
 			{
 				if (errno != ENOENT)	/* file did not disappear? then something is very wrong */
-					error_exit("match_files: stat error on %s\n", cur_dir);
+					error_exit(TRUE, FALSE, "Error while invoking stat() %s.\n", cur_dir);
 			}
 			(*isdir)[nfound] = S_ISDIR(finfo.st_mode)?1:0;
 
@@ -480,7 +492,7 @@ int match_files(char *search_for, char **path, char ***found, char **isdir)
 	}
 
 	if (closedir(dir) == -1)
-		error_exit("match_files: closedir failed\n");
+		error_exit(TRUE, FALSE, "closedir() failed\n");
 
 	/*	qsort( (void *)list, (size_t)nfound, sizeof(char *), compare_filenames); */
 	for(s1=0; s1<(nfound - 1); s1++)
@@ -509,14 +521,15 @@ int match_files(char *search_for, char **path, char ***found, char **isdir)
 
 void setup_for_childproc(int fd, char close_fd_0, char *term)
 {
-	char *dummy = (char *)mymalloc(strlen(term) + 6, "setup_for_childproc: TERM env var");
+	int term_len = strlen(term) + 5;
+	char *dummy = (char *)mymalloc(term_len + 1);
 
-	if (close_fd_0) if (-1 == myclose(0)) error_exit("setup_for_childproc: close failed\n");
-	if (-1 == myclose(1)) error_exit("setup_for_childproc: close failed\n");
-	if (-1 == myclose(2)) error_exit("setup_for_childproc: close failed\n");
-	if (close_fd_0) if (-1 == dup(fd)) error_exit("setup_for_childproc: dup failed\n");
-	if (-1 == dup(fd)) error_exit("setup_for_childproc: dup failed\n");
-	if (-1 == dup(fd)) error_exit("setup_for_childproc: dup failed\n");
+	if (close_fd_0) if (-1 == myclose(0)) error_exit(TRUE, FALSE, "close() failed\n");
+	if (-1 == myclose(1)) error_exit(TRUE, FALSE, "close() failed\n");
+	if (-1 == myclose(2)) error_exit(TRUE, FALSE, "close() failed\n");
+	if (close_fd_0) if (-1 == mydup(fd)) error_exit(TRUE, FALSE, "dup() failed\n");
+	if (-1 == mydup(fd)) error_exit(TRUE, FALSE, "dup() failed\n");
+	if (-1 == mydup(fd)) error_exit(TRUE, FALSE, "dup() failed\n");
 
 	/*
 	 * not doing this: it also clears the 'PATH'
@@ -532,7 +545,8 @@ void setup_for_childproc(int fd, char close_fd_0, char *term)
 	 */
 
 	/* set terminal */
-	sprintf(dummy, "TERM=%s", term);
+	assert(term != NULL);
+	snprintf(dummy, term_len, "TERM=%s", term);
 	if (putenv(dummy) == -1)
 		fprintf(stderr, "setup_for_childproc: Could not set TERM environment-variable (%s): %s (%d)\n", dummy, strerror(errno), errno);
 
@@ -543,7 +557,7 @@ int open_null(void)
 {
 	int fd = myopen("/dev/null", O_RDWR);
 	if (fd == -1)
-		error_exit("open_null: Failed to open /dev/null\n");
+		error_exit(TRUE, FALSE, "Failed to open file /dev/null.\n");
 
 	return fd;
 }
@@ -567,14 +581,11 @@ char * find_most_recent_file(char *filespec, char *cur_file)
 	unsigned int loop;
 	time_t prev_ts = (time_t)0;
 
-	LOG("find file for %s\n", filespec);
-
 	/* get timestamp of previous file */
 	if (cur_file)
 	{
 		if (file_info(cur_file, NULL, TT_MTIME, &prev_ts, NULL) == -1)
 		{
-			LOG("could not get fileinfo for %s\n", cur_file);
 			prev_ts = (time_t)0;
 		}
 	}
@@ -583,14 +594,11 @@ char * find_most_recent_file(char *filespec, char *cur_file)
 #if defined(__APPLE__) || defined(__CYGWIN__)
 	if (glob(filespec, GLOB_ERR | GLOB_NOSORT, NULL, &files) != 0)
 #else
-		if (glob(filespec, GLOB_ERR | GLOB_NOSORT | GLOB_NOESCAPE, NULL, &files) != 0)
+	if (glob(filespec, GLOB_ERR | GLOB_NOSORT | GLOB_NOESCAPE, NULL, &files) != 0)
 #endif
-		{
-			LOG("glob failed %d\n", errno);
-			return NULL;
-		}
-
-	LOG("number of files found: %d\n", files.gl_pathc);
+	{
+		return NULL;
+	}
 
 	/* see if any of them is more recent than the current one */
 	for(loop=0; loop<files.gl_pathc; loop++)
@@ -604,7 +612,6 @@ char * find_most_recent_file(char *filespec, char *cur_file)
 		/* get the modificationtime of this found file */
 		if (file_info(files.gl_pathv[loop], NULL, TT_MTIME, &new_ts, NULL) == -1)
 		{
-			LOG("loop: failed fetching info on file %s: %d\n", files.gl_pathv[loop], errno);
 			new_ts = (time_t)0;
 		}
 
@@ -619,7 +626,7 @@ char * find_most_recent_file(char *filespec, char *cur_file)
 	/* found a file? then remember the filename */
 	if (selected_file != NULL)
 	{
-		selected_file = mystrdup(selected_file, "find_most_recent_file: new file filename");
+		selected_file = mystrdup(selected_file);
 	}
 
 	globfree(&files);
@@ -662,7 +669,7 @@ int mydup(int old_fd)
 			if (errno == EINTR)
 				continue;
 
-			error_exit("mydup: dup() failed\n");
+			error_exit(TRUE, FALSE, "dup() failed\n");
 		}
 
 		break;
@@ -680,16 +687,332 @@ char * term_t_to_string(term_t term)
 
 		case TERM_XTERM:
 			return "xterm";
+
+		case TERM_IGNORE:
+			return "dumb";
 	}
 
 	return "dumb";
 }
 
-void get_now_ts(char *format_str, char *dest, int dest_size)
+void double_ts_to_str(dtime_t ts, char *format_str, char *dest, int dest_size)
 {
-	time_t now = time(NULL);
+	time_t now = (time_t)ts;
 	struct tm *ptm = localtime(&now);
-	if (!ptm) error_exit("get_now_ts: localtime() failed\n");
+	if (!ptm) error_exit(TRUE, FALSE, "localtime() failed\n");
+
+	assert(ts > 0);
+	assert(dest_size > 0);
 
 	(void)strftime(dest, dest_size, format_str, ptm);
 }
+
+void get_now_ts(char *format_str, char *dest, int dest_size)
+{
+	double_ts_to_str(get_ts(), format_str, dest, dest_size);
+}
+
+void duplicate_re_array(re *pre_in, int n_rein, re **pre_out, int *n_reout)
+{
+	int loop, old_n = *n_reout;
+
+	assert(n_rein >= 0);
+
+	*n_reout += n_rein;
+	if (*n_reout)
+	{
+		*pre_out = (re *)myrealloc(*pre_out, (*n_reout) * sizeof(re));
+
+		for(loop=0; loop<n_rein; loop++)
+		{
+			memcpy(&(*pre_out)[loop + old_n], &pre_in[loop], sizeof(re));
+
+			memset(&(*pre_out)[loop + old_n].regex, 0x00, sizeof(regex_t));
+			(*pre_out)[loop + old_n].regex_str = mystrdup(pre_in[loop].regex_str);
+			compile_re(&(*pre_out)[loop + old_n].regex, (*pre_out)[loop + old_n].regex_str);
+
+			if (pre_in[loop].cmd)
+				(*pre_out)[loop + old_n].cmd = mystrdup(pre_in[loop].cmd);
+		}
+	}
+	else
+	{
+		*pre_out = NULL;
+	}
+}
+
+void duplicate_es_array(strip_t *pes_in, int n_esin, strip_t **pes_out, int *n_esout)
+{
+	int loop, old_n = *n_esout;
+
+	assert(n_esin >= 0);
+
+	*n_esout += n_esin;
+	*pes_out = (strip_t *)myrealloc(*pes_out, (*n_esout) * sizeof(strip_t));
+
+	for(loop=0; loop<n_esin; loop++)
+	{
+		memcpy(&(*pes_out)[loop + old_n], &pes_in[loop], sizeof(strip_t));
+
+		memset(&(*pes_out)[loop + old_n].regex, 0x00, sizeof(regex_t));
+		if (pes_in[loop].type != STRIP_TYPE_COLUMN && pes_in[loop].type != STRIP_TYPE_RANGE)
+		{
+			(*pes_out)[loop + old_n].regex_str = mystrdup(pes_in[loop].regex_str);
+
+			compile_re(&(*pes_out)[loop + old_n].regex, (*pes_out)[loop + old_n].regex_str);
+		}
+
+		if (pes_in[loop].del)
+			(*pes_out)[loop + old_n].del = mystrdup(pes_in[loop].del);
+	}
+}
+
+int find_filterscheme(char *name)
+{
+	int loop;
+
+	for(loop=0; loop<n_fs; loop++)
+	{
+		if (strcmp(pfs[loop].fs_name, name) == 0)
+			return loop;
+	}
+
+	return -1;
+}
+
+int find_editscheme(char *name)
+{
+	int loop;
+
+	for(loop=0; loop<n_es; loop++)
+	{
+		if (strcmp(pes[loop].es_name, name) == 0)
+			return loop;
+	}
+
+	return -1;
+}
+
+void compile_re(regex_t *whereto, char *what)
+{
+	/* compile & store regular expression */
+	int rc = regcomp(whereto, what, REG_EXTENDED);
+	if (rc != 0)
+		error_exit(FALSE, FALSE, "Failed to compile regular expression '%s'.\nReason: %s\n", what, convert_regexp_error(rc, whereto));
+}
+
+void grow_mem_if_needed(char **what, int *cur_len, int requested_len)
+{
+	char changed = 0;
+
+	assert(requested_len > 0);
+
+	while(*cur_len < requested_len)
+	{
+		changed = 1;
+
+		if (*cur_len)
+			(*cur_len) *= 2;
+		else
+			*cur_len = 128;
+	}
+
+	if (changed)
+		*what = myrealloc(*what, *cur_len);
+}
+
+int READ(int fd, char *where_to, int max_len, char *for_whom)
+{
+	assert(max_len > 0);
+
+	for(;;)
+	{
+		int rc = read(fd, where_to, max_len);
+		if (rc >= 0)
+			return rc;
+
+		if (errno != EINTR && errno != EAGAIN)
+			error_exit(TRUE, FALSE, "Error writing to fd for %s.\n", for_whom);
+	}
+}
+
+int get_value_arg(char *par, char *string, valcheck_t check)
+{
+	long int result;
+	int len, loop;
+	int multiplier = 1;
+
+	if (!string)
+		error_exit(FALSE, FALSE, "%s needs a parameter.\n", par);
+
+	len = strlen(string);
+
+	for(loop=0; loop<len; loop++)
+	{
+		if (!isdigit(string[loop]))
+		{
+			char dummy = toupper(string[loop]);
+
+			if (toupper(string[loop + 1]) == 'B' && (dummy == 'K' || dummy == 'M' || dummy == 'G'))
+			{
+				if (dummy == 'K')
+					multiplier = 1024;
+				else if (dummy == 'M')
+					multiplier = 1024 * 1024;
+				else if (dummy == 'G')
+					multiplier = 1024 * 1024 * 1024;
+
+				break;
+			}
+			else
+				error_exit(FALSE, FALSE, "%s needs a value as parameter.\n", par);
+		}
+	}
+
+	result = strtol(string, NULL, 10);
+
+	if (result > INT_MAX || result == LONG_MIN || result == LONG_MAX)
+		error_exit(FALSE, FALSE, "Value %s for parameter %s is too large.\n", string, par);
+
+	result *= multiplier;
+
+	if (check == VAL_ZERO_POSITIVE)
+	{
+		if (result < 0)
+			error_exit(FALSE, FALSE, "Value for %s must be >= 0.\n", par);
+	}
+	else if (check == VAL_POSITIVE_NOT_1)
+	{
+		if (result < 0 || result == 1)
+			error_exit(FALSE, FALSE, "Value for %s must be >= 0 and must not be 1.\n", par);
+	}
+	else if (check == VAL_POSITIVE)
+	{
+		if (result <= 0)
+			error_exit(FALSE, FALSE, "Value for %s must be > 0.\n", par);
+	}
+	else
+	{
+		assert(0);
+	}
+
+	return (int)result;
+}
+
+void add_to_iat(int_array_t *piat, int element)
+{
+	if (piat -> n == piat -> size)
+	{
+		piat -> size = USE_IF_SET(piat -> size, 8) * 2;
+		piat -> elements = (int *)myrealloc(piat -> elements, piat -> size * sizeof(int));
+	}
+
+	(piat -> elements)[piat -> n] = element;
+	(piat -> n)++;
+}
+
+int get_iat_size(int_array_t *piat)
+{
+	return piat -> n;
+}
+
+void init_iat(int_array_t *piat)
+{
+	piat -> elements = NULL;
+	piat -> size = piat -> n = 0;
+}
+
+void free_iat(int_array_t *piat)
+{
+	myfree(piat -> elements);
+
+	init_iat(piat);
+}
+
+int get_iat_element(int_array_t *piat, int index)
+{
+	assert(index < piat -> n);
+
+	return (piat -> elements)[index];
+}
+
+char *gethome(char *user)
+{
+	struct passwd *pw;
+	if (user)
+		pw = getpwnam(user);
+	else
+		pw = getpwuid(getuid());
+	return mystrdup(pw->pw_dir);
+}
+
+char *myrealpath(char *in)
+{
+	char *home, *pin;
+	int home_len;
+	char *pout;
+
+	if (in[0] != '~')
+		return mystrdup(in);
+
+	if (in[1] != '/')
+	{
+		char *user;
+		char *slash = strchr(in, '/');
+		int len;
+
+		if (slash)
+		{
+			len = (int)((slash - in) - 1);
+			pin = slash + 1;
+		}
+		else
+		{
+			len = strlen(in) - 1;
+			pin = in + len + 1;
+		}
+
+		user = (char *)mymalloc(len + 1);
+		memcpy(user, &in[1], len);
+		user[len] = 0x00;
+
+		home = gethome(user);
+
+		myfree(user);
+	}
+	else
+	{
+		home = gethome(NULL);
+		pin = in + 1;
+	}
+
+	if (!home)
+		error_exit(FALSE, FALSE, "Cannot expand path %s:\nhome directory not found.\n", in);
+
+	home_len = strlen(home);
+	pout = (char *)mymalloc(home_len + strlen(in) + 1);
+
+	sprintf(pout, "%s/%s", home, pin);
+
+	myfree(home);
+
+	return pout;
+}
+
+void str_add(char **to, const char *what, ...)
+{
+	int len_to = *to ? strlen(*to) : 0;
+	char buffer[4096] = { 0 };
+	int len_what = 0;
+
+        va_list ap;
+
+	/* FIXME: should vasprintf here: does other unixes have it? */
+        va_start(ap, what);
+        len_what = vsnprintf(buffer, sizeof buffer, what, ap);
+        va_end(ap);
+
+	*to = (char *)realloc(*to, len_to + len_what + 1);
+
+	memcpy(&(*to)[len_to], buffer, len_what + 1);
+}
